/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import java.util.List;

import org.conqat.lib.commons.string.StringUtils;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.policy.RemoveEditPolicy;
import org.fortiss.tooling.base.ui.layout.constants.RecommendedLayoutConstants;
import org.fortiss.tooling.base.ui.layout.util.LayoutDataUtils;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

/**
 * The base class for graphical edit parts which encapsulate nodes (i.e. the
 * main modeling elements). All of them have a name label and an activation
 * mode.
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: D6B33EFB704B0CA151E1C945B5BC48C6
 */
public abstract class NodeEditPartBase<T extends ILayoutedModelElement & INamedCommentedElement>
		extends PositionedEditPartBase<T> {

	/** The label used for the name. */
	protected final Label nameLabel = new Label();

	/** Constructor. */
	protected NodeEditPartBase(T modelObject) {
		super(modelObject);
	}

	/** Creates the node edit part. */
	private void createLabelFigure() {
		Rectangle rect = LayoutDataUtils.getNodeBounds(modelElement);
		int inset = 2 * RecommendedLayoutConstants.RECTANGULAR_SHAPE_INSETS;
		rect = new Rectangle(rect.x + inset, rect.y + inset, rect.width - 2
				* inset, rect.height - 2 * inset);
		nameLabel.setForegroundColor(getForegroundColor());
		nameLabel.setFont(RecommendedLayoutConstants.TITLE_FONT);
		nameLabel.setTextAlignment(PositionConstants.LEFT
				| PositionConstants.TOP);
		nameLabel.setTextPlacement(PositionConstants.EAST);
		nameLabel.setIcon(IModelElementHandlerService.INSTANCE
				.getModelElementHandler(modelElement).getIcon());
		getLayer(LayerConstants.GUIDE_LAYER).add(nameLabel);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the connectors as children.
	 */
	@Override
	protected List<?> getModelChildren() {
		return modelElementHandler.getConnectors(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determinePosition() {
		return LayoutDataUtils.getNodeBounds(modelElement);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Implements the component role for deletion.
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RemoveEditPolicy());
	}

	/** {@inheritDoc} */
	@Override
	public void performRequest(Request req) {
		if (req.getType() == RequestConstants.REQ_OPEN) {
			IModelEditorBindingService.INSTANCE.openInEditor(modelElement);
		}
		super.performRequest(req);
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();

		nameLabel.setText(modelElement.getName());
		String tooltipText = StringUtils.wrapLongLines(modelElement.getName()
				+ ": " + modelElement.getComment(), 100);
		baseFigure.setToolTip(new Label(tooltipText));
	}
}
