/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.SelectionRequest;
import org.eclipse.gef.tools.ConnectionDragCreationTool;
import org.eclipse.gef.tools.DragEditPartsTracker;
import org.eclipse.swt.graphics.Color;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;

/**
 * The base class for all edit parts which are positioned using x/y coordinates.
 * Additionally this class introduces the marker figure.
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: D6B33EFB704B0CA151E1C945B5BC48C6
 */
public abstract class PositionedEditPartBase<T extends ILayoutedModelElement>
		extends LayoutedEditPartBase<T> implements NodeEditPart {
	/** The base figure of this node edit part. */
	protected IFigure baseFigure;

	/** Constructor. */
	protected PositionedEditPartBase(T modelElement) {
		super(modelElement);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected final IFigure createFigure() {
		baseFigure = createBaseFigure();
		// createLabelFigure();
		// TODO (FH): implement marker support
		// figure.add(markerFigure);

		return baseFigure;
	}

	/** Creates the outer base figure. */
	protected abstract IFigure createBaseFigure();

	/**
	 * Returns the position and size this edit part's main figure should have in
	 * the layout.
	 */
	protected abstract Rectangle determinePosition();

	/**
	 * Returns the foreground color used.
	 */
	protected abstract Color getForegroundColor();

	/**
	 * Returns the background color used.
	 */
	protected abstract Color getBackgroundColor();

	/**
	 * Returns the default connection anchor for this port (does not depend on
	 * type of connection).
	 */
	protected abstract ConnectionAnchor defaultConnectionAnchor();

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		// we have to make sure, that we are registered as constraint with our
		// parent (if we have one)
		// if (getParent() instanceof GraphicalEditPart) {
		// ((GraphicalEditPart) getParent()).getFigure().setConstraint(
		// getFigure(), determinePosition());
		// }

		super.refresh();
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		baseFigure.setBounds(determinePosition());
		baseFigure.setForegroundColor(getForegroundColor());
		baseFigure.setBackgroundColor(getBackgroundColor());
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getSourceConnectionAnchor(
			ConnectionEditPart connection) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getTargetConnectionAnchor(
			ConnectionEditPart connection) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public DragTracker getDragTracker(Request request) {
		if (request instanceof SelectionRequest) {
			SelectionRequest sr = (SelectionRequest) request;
			if (sr.getLastButtonPressed() == 1
					&& (sr.isAltKeyPressed() || sr.isControlKeyPressed())) {
				return new ConnectionDragCreationTool();
			} else if (sr.getLastButtonPressed() == 1) {
				return new DragEditPartsTracker(this);
			}
		}
		return null;
	}
}